#!/usr/bin/env wish

#******************************
# 
# osrechenspiel
# Version 2014-12-20
# Copyright (C) 2014 Oliver Scholl < tcltk@posteo.de > < www.tcltk.info >
#
# Requirement:
# tcl 8.6 or higher
# tk 8.6 or higher
#
# Recommendation:
# 
# Beschreibung des Programms:
# Das Programm übt das Kopfrechnen in den vier Grundrechenarten
# plus, minus, mal und geteilt. Die Schwierigkeit (Zahlenraum,
# Rechenzeit) kann individuell eingestellt werden.
#
# This program is free software; you can redistribute it and/or 
# modify it under the terms of the GNU General Public License as 
# published by the Free Software Foundation; either version 2 of 
# the license, or any later version.
#
# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of 
# merchantability or fitness for a particular purpose. 
# See the GNU General Public License for mor details.
#
#*************************************

package require msgcat

set ProgrammOrdner ".osProgramme"
set ProgrammName "osRechenspiel"
set ProgrammVersion "2014-12-20"
set ProgrammTitel "Rechenspiel"

set Konf(Skriptname) [info script]
if {[file type $Konf(Skriptname)] == "link"} {
	set Konf(Skriptname) [file readlink $Konf(Skriptname)]
}

source [file join [file dirname $Konf(Skriptname)] basis.tcl]

#ttk::style theme use default 
font create MeineSchrift -family Helvetica -size 16
font create MeineSchriftKlein -family Helvetica -size 10
ttk::style configure TButton -font MeineSchriftKlein
ttk::style configure TLabel -font MeineSchriftKlein

# Globale Variablen (alphabetisch sortiert)
set Glob(AnzahlRichtig) 0 ; # Anzahl richtig gelöster Aufgaben
set Glob(AnzahlFalsch) 0 ; # Anzahl falsch gelöster Aufgaben
set Glob(BalkenStopp) 0 ; # Boolean. Wenn 1, dann stoppt die Bewegung des Fortschrittsbalkens
set Glob(TextErgebnis) "" ; # Anzeigetext
set Glob(FigurComputer2) "" ; # Pfadname der Computerfigur
set Glob(FigurComputerStopp) 0 ; # Boolean. Wenn 1, dann stoppt die Bewegung der Figur des Computers
set Glob(FigurSpieler2) "" ; # Pfadname der Spielerfigur
set Glob(FigurZiel2) "" ; # Pfadname der Zielfigur
set Glob(FigurSpielerY) 5 ; # y-Koordinate für die Spielerfigur
set Glob(FigurComputerY) 40 ; # y-Koordinate für die Computerfigur
set Glob(FigurZielY) 5 ; # y-Koordinate für die Zielfigur
set Glob(GesamtstreckeMax) 700 ; # Maximale Streckenlänge (in Pixel), die die Figuren laufen sollen (damit das Anwendungsfenster nicht zu breit wird)
set Glob(Gesamtstrecke) 0 ; # Tatsächliche Streckenlänge (in Pixel), die die Figuren laufen sollen
set Glob(Loesung) "" ; # beim Erstellen der Aufgabe wird die Lösung gleich mitbestimmt
set Glob(NaechsteAufgabe) 0 ; # Boolean. Wenn der Benutzer die Return-Taste drückt, wird diese Taste mehrmals ausgeführt, somit auch die Prozedur ErgebnisPruefen. Mit dieser Variablen wird verhindert, dass die Prozedur mehrmals ausgeführt wird.
set Glob(Operator1) "+" ; # Rechenzeichen in der Rechenaufgabe
set Glob(Operator2) "=" ; # Zeichen vor dem Ergebnis der Rechenaufgabe
set Glob(Rechenzeit) "" ; # Verbrauchte Rechenzeit des Spielers
set Glob(RechenzeitMax) 5000 ; # Maximale Rechenzeit je Aufgabe in Millisekunden.
set Glob(TextPunkte) "" ; # Anzeigetext
set Glob(SchrittweiteFigur) 0 ; # Schrittweite (in Pixel) während der Animation der Figuren
set Glob(Startzeit) 0 ; # Zeitstempel, wenn der Spieler mit einer neuen Rechenaufgabe beginnt
set Glob(StufenMax) 9 ; # maximale Anzahl an Stufen (Level)
set Glob(Teilstrecke) 100 ; # Teilstrecke (in Pixel), die die Figuren während einer Rechenaufgabe laufen sollen
set Glob(xStart) 0 ; # x-Position, an der die Figur startet
set Glob(Zahl1) "" ; # 1. Zahl der Rechenaufgabe
set Glob(Zahl2) "" ; # 2. Zahl der Rechenaufgabe
set Glob(Zahl3) "" ; # 3. Zahl der Rechenaufgabe (= Ergebnis der Rechenaufgabe)
set Glob(ListeAlteErgebnisse,0) {} ;

# **** Prozeduren, die keine anderen Prozeduren aufrufen ***

proc Aufgabentext {Anzahl} {
	return [::msgcat::mc "Loese %d Aufgaben. Kommst Du weiter als die Schildkroete?" $Anzahl]
}

proc AnimationsdatenEinstellen {Teilstrecke GesamtstreckeMax Anzahl RechenzeitMax} {
	# ermittelt die Gesamtstrecke, Teilstrecke und Schrittweite für die Animation der Figuren
	
	set Gesamtstrecke [expr $Teilstrecke*$Anzahl]
	if {$Gesamtstrecke > $GesamtstreckeMax} {
		set Gesamtstrecke $GesamtstreckeMax
		set Teilstrecke [expr $Gesamtstrecke/$Anzahl]
		set Gesamtstrecke [expr $Teilstrecke*$Anzahl]
	}

	set SchrittweiteFigur [expr $Teilstrecke/double($RechenzeitMax)]
	
	lappend Rueckgabe $Gesamtstrecke $Teilstrecke $SchrittweiteFigur
	return $Rueckgabe
}

proc AnimationZeichnen {Figur Gesamtstrecke yKoordinate} {
	# zeichnet die Gesamtstrecke und positioniert die Zielfigur
	
	.can configure -width [expr $Gesamtstrecke + 80]
	.can moveto $Figur [expr $Gesamtstrecke + 40] $yKoordinate
}

proc FigurBewegen {Figur SchrittweiteFigur Startzeit RechenzeitMax yKoordinate} {
	# zieht die Computer- bzw. Spielerfigur gemäß der Restzeit
  
	set AktuelleZeit [clock clicks -milliseconds]
	set Rechenzeit [expr $AktuelleZeit - $Startzeit]
	set xStart [expr int([lindex [.can coords $Figur] 0])]
	set xMax [expr $xStart+int(($RechenzeitMax-$Rechenzeit)*double($SchrittweiteFigur))]
	for {set x $xStart} {$x <= $xMax} {incr x} {
		.can moveto $Figur $x $yKoordinate
		update idletask
		after 20
	}
}

proc Programmende {} {
	# beendet das Programm
	
	global Glob ; # nur lesender Zugriff
	
	EinstellungenWerteUebernehmen
	KonfigurationSpeichern $Glob(StufenMax)
	exit
}

proc PunkteAnzeigen {Spielende AnzahlRichtig} {
	# aktualisiert den Punktestand
  
	set Text1 ""
	set Text2 ""
	set Text3 ""
  
	if {$AnzahlRichtig == 1} {
		set Text2 [::msgcat::mc "%d Punkt" $AnzahlRichtig]
	} else {
		set Text2 [::msgcat::mc "%d Punkte" $AnzahlRichtig]
	}	 

	if {$Spielende == 1} {
		set Text1 [::msgcat::mc "Das Spiel ist zu Ende. Du hast "]
		set Text3 [::msgcat::mc " erreicht."] 
	}

	set Rueckgabe $Text1$Text2$Text3
	return $Rueckgabe
}

proc EinstellungenWerteSetzen {} {
	# aktualisiert die Werte im Einstellungsdialog

	global Konf ; # lesender und schreibender Zugriff

	for {set i 0} {$i <= 3} {incr i} {
		set Konf(Einstellungen,$i,JaNein) $Konf($Konf(Einstellungen,Stufe),$i,JaNein)
		set Konf(Einstellungen,$i,Von) $Konf($Konf(Einstellungen,Stufe),$i,Von)
		set Konf(Einstellungen,$i,Bis) $Konf($Konf(Einstellungen,Stufe),$i,Bis)
		if {$i >= 2} {
			set Konf(Einstellungen,$i,Variante) $Konf($Konf(Einstellungen,Stufe),$i,Variante)
		}
		set Konf(Einstellungen,$i,FrageNachZahl1) $Konf($Konf(Einstellungen,Stufe),$i,FrageNachZahl1)
		set Konf(Einstellungen,$i,FrageNachZahl2) $Konf($Konf(Einstellungen,Stufe),$i,FrageNachZahl2)
		set Konf(Einstellungen,$i,FrageNachZahl3) $Konf($Konf(Einstellungen,Stufe),$i,FrageNachZahl3)
		set Konf(Einstellungen,$i,Zeit) $Konf($Konf(Einstellungen,Stufe),$i,Zeit)
	}    
	set Konf(Einstellungen,Anzahl) $Konf($Konf(Einstellungen,Stufe),Anzahl)
}

proc EinstellungenWerteUebernehmen {} {
    # übernimmt die Werte aus dem Einstellungsdialog
    
	global Konf ; # lesender und schreibender Zugriff
 
	for {set i 0} {$i <= 3} {incr i} {
		set Konf($Konf(Einstellungen,Stufe),$i,JaNein) $Konf(Einstellungen,$i,JaNein)
		set Konf($Konf(Einstellungen,Stufe),$i,Von) $Konf(Einstellungen,$i,Von)
		set Konf($Konf(Einstellungen,Stufe),$i,Bis) $Konf(Einstellungen,$i,Bis)
		if {$i >= 2} {
			set Konf($Konf(Einstellungen,Stufe),$i,Variante) $Konf(Einstellungen,$i,Variante)
		}
		set Konf($Konf(Einstellungen,Stufe),$i,FrageNachZahl1) $Konf(Einstellungen,$i,FrageNachZahl1)
		set Konf($Konf(Einstellungen,Stufe),$i,FrageNachZahl2) $Konf(Einstellungen,$i,FrageNachZahl2)
		set Konf($Konf(Einstellungen,Stufe),$i,FrageNachZahl3) $Konf(Einstellungen,$i,FrageNachZahl3)
		set Konf($Konf(Einstellungen,Stufe),$i,Zeit) $Konf(Einstellungen,$i,Zeit)
	}    
	set Konf($Konf(Einstellungen,Stufe),Anzahl) $Konf(Einstellungen,Anzahl)
}

proc AufgabeFelderFreigeben {i Zahl1 Zahl2 Zahl3} {
	# aktiviert das zu berechnende Zahlenfeld
	
	# zu übergebende Variablen:
	# i = Aufgabentyp (0=Plus, 1=Minus, 2=Mal, 3=Geteilt)
	
	global Konf ; # nur lesender Zugriff
	
	set Loesung ""
	
	.frAufgabe.enZahl1 configure -state readonly
	.frAufgabe.enZahl2 configure -state readonly
	.frAufgabe.enZahl3 configure -state readonly
	
	set OK 0
	while {$OK == 0} {
		set Zufallszahl [expr int(3*rand())]
		if {($Zufallszahl == 0 && $Konf(Einstellungen,$i,FrageNachZahl1) == 1) || ($Zufallszahl == 1 && $Konf(Einstellungen,$i,FrageNachZahl2) == 1) || ($Zufallszahl == 2 && $Konf(Einstellungen,$i,FrageNachZahl3) == 1)} {
			set OK 1
		}
	}
	
	switch $Zufallszahl {
		0 {.frAufgabe.enZahl1 configure -state normal;set Loesung $Zahl1; set Zahl1 ""; focus .frAufgabe.enZahl1}
		1 {.frAufgabe.enZahl2 configure -state normal;set Loesung $Zahl2; set Zahl2 "";focus .frAufgabe.enZahl2}
		2 {.frAufgabe.enZahl3 configure -state normal;set Loesung $Zahl3; set Zahl3 "";focus .frAufgabe.enZahl3}
		default {.frAufgabe.enZahl3 configure -state normal;set Loesung $Zahl3; set Zahl3 "";focus .frAufgabe.enZahl3}
	}
	
	lappend Rueckgabe $Zahl1 $Zahl2 $Zahl3 $Loesung
	return $Rueckgabe
}


wm protocol . WM_DELETE_WINDOW {
	.frButton.btEnde invoke
}

proc DialogEinstellungenOKButtonKlick {} {
	# wird ausgeführt, wenn im Dialog Einstellungen der OK-Button angeklickt wird
	
	global Konf ; # nur lesender Zugriff
	global Glob ; # lesender und schreibender Zugriff

	EinstellungenWerteUebernehmen
	KonfigurationSpeichern $Glob(StufenMax)

	set Glob(TextPunkte) [Aufgabentext $Konf(Einstellungen,Anzahl)]
	
	set Rueckgabe [AnimationsdatenEinstellen $Glob(Teilstrecke) $Glob(GesamtstreckeMax) $Konf(Einstellungen,Anzahl) $Glob(RechenzeitMax)]
	lassign $Rueckgabe Glob(Gesamtstrecke) Glob(Teilstrecke) Glob(SchrittweiteFigur)
	
	AnimationZeichnen $Glob(FigurZiel2) $Glob(Gesamtstrecke) $Glob(FigurZielY)
	grab release .einstellungen
	destroy .einstellungen
}

proc ZahlZiehen {Von Bis Grenze1 Grenze2 Faktor} {
	# generiert eine Menge von Zahlen im Bereich Von bis Bis
	# Zahlen, die kleiner Grenze1 sind, werden einfach in die Zahlenmenge aufgenommen
	# Zahlen, die größer Grenze2 sind, werden einfach in die Zahlenmenge aufgenommen
	# Zahlen, die zwischen Grenze1 und Grenze2 sind, werden mehrfach (Faktor) in die Zahlenmenge aufgenommen
	# Danach wird zufällig eine Zahl aus der Zahlenmenge gezogen
	# Rückgabe: eine zufällige Zahl
	
	set Liste {}
	for {set i $Von} {$i <= $Bis} {incr i} {
		if {$i < $Grenze1 || $i > $Grenze2} {
			lappend Liste $i
		} else {
			for {set j 1} {$j <= $Faktor} {incr j} {
				lappend Liste $i
			}
		}
	}
	set AnzahlElemente [llength $Liste]
	set Zufallszahl [expr int($AnzahlElemente*rand())]
	set Element [lindex $Liste $Zufallszahl]
	return $Element
}

# **** Prozeduren, die andere Prozeduren aufrufen ***

proc DialogEinstellungen {} {
	# Einstellungsdialog anzeigen
	
	global Konf
	global Glob

	set ListeStufen {}
	for {set i 1} {$i <= $Glob(StufenMax)} {incr i} {
		lappend ListeStufen $i
	}

	toplevel .einstellungen
	wm title .einstellungen [::msgcat::mc "Einstellungen" ]
	wm resizable .einstellungen 0 0

	ttk::label .einstellungen.lbStufe -text [::msgcat::mc "Stufe:"]
	ttk::combobox .einstellungen.combStufe -state readonly -width 5 -values $ListeStufen -textvariable Konf(Einstellungen,Stufe)
	ttk::label .einstellungen.lbZahlenraum -text [::msgcat::mc "Zahlenraum:"]
	ttk::label .einstellungen.lbFrage -text [::msgcat::mc "gesucht wird:"]
	ttk::label .einstellungen.lbRechenzeit -text [::msgcat::mc "Rechenzeit pro Aufgabe:"]
	
	set Typ {}
	lappend Typ [msgcat::mc "Plus"]
	lappend Typ [msgcat::mc "Minus"]
	lappend Typ [msgcat::mc "Mal"]
	lappend Typ [msgcat::mc "Geteilt"]
	for {set i 0} {$i <= 3} {incr i} {
		ttk::checkbutton .einstellungen.cbTyp${i}JaNein -text [lindex $Typ $i] -variable Konf(Einstellungen,$i,JaNein)
		ttk::label .einstellungen.lbTyp${i}Von -text [::msgcat::mc "von"]
		ttk::entry .einstellungen.enTyp${i}Von -width 5 -textvariable Konf(Einstellungen,$i,Von)
		ttk::label .einstellungen.lbTyp${i}Bis -text [::msgcat::mc "bis"]
		ttk::entry .einstellungen.enTyp${i}Bis -width 5 -textvariable Konf(Einstellungen,$i,Bis)
		
		if {$i >= 2} {
			ttk::checkbutton .einstellungen.cbTyp${i}Variante -text [::msgcat::mc "schwieriger"] -variable Konf(Einstellungen,$i,Variante)
		}
		
		ttk::checkbutton .einstellungen.cbTyp${i}Zahl1 -text [::msgcat::mc "1. Zahl"] -variable Konf(Einstellungen,$i,FrageNachZahl1)
		ttk::checkbutton .einstellungen.cbTyp${i}Zahl2 -text [::msgcat::mc "2. Zahl"] -variable Konf(Einstellungen,$i,FrageNachZahl2)
		ttk::checkbutton .einstellungen.cbTyp${i}Zahl3 -text [::msgcat::mc "Ergebnis"] -variable Konf(Einstellungen,$i,FrageNachZahl3)
		ttk::label .einstellungen.lbTyp${i}Zeit -text [::msgcat::mc "Zeit:"]
		ttk::entry .einstellungen.enTyp${i}Zeit -width 5 -textvariable Konf(Einstellungen,$i,Zeit)
		ttk::label .einstellungen.lbTyp${i}ZeitSekunden -text [::msgcat::mc "Sekunden"]
	}
	ttk::label .einstellungen.lbAnzahl -text [msgcat::mc "Anzahl Aufgaben:"] 
	ttk::entry .einstellungen.enAnzahl -width 5 -textvariable Konf(Einstellungen,Anzahl)
	ttk::button .einstellungen.btOK -text [::msgcat::mc "OK"] -command DialogEinstellungenOKButtonKlick

	set Text "Hilfe:"
	set Text "${Text}\nIn der ersten Spalte kannst Du auswählen, welche der vier Grundrechenarten Du üben möchtest. Du kannst auch mehrere Rechenarten anklicken."
	set Text "${Text}\nDanach legst Du den Zahlenraum fest, das heißt, wie groß die Zahlen in der Rechnung sein sollen."
	set Text "${Text}\nDann gibst Du die Sekunden ein, wie lange Du an einer Aufgabe längstens rechnen willst."
	set Text "${Text}\nUnd zum Schluss wählst Du, welche Teile der Aufgabe zu berechnen sind, also die erste oder zweite Zahl oder das Ergebnis. Du kannst auch mehrere Kästchen anklicken."

	ttk::label .einstellungen.lbHilfe -wraplength 700 -text $Text -anchor e -justify left
	ttk::label .einstellungen.lbAbstand1 -text ""
	ttk::label .einstellungen.lbAbstand2 -text ""


	grid .einstellungen.lbStufe -row 0 -column 0 -sticky w -pady 2
	grid .einstellungen.combStufe -row 0 -column 1 -sticky w -pady 2 -padx 2 -columnspan 2

	grid .einstellungen.lbAnzahl -row 1 -column 0 -pady 2
	grid .einstellungen.enAnzahl -row 1 -column 1 -sticky w -pady 2

	grid .einstellungen.lbZahlenraum -row 2 -column 2 -sticky w -pady 2 -padx 2 -columnspan 4
	grid .einstellungen.lbRechenzeit -row 2 -column 7 -sticky w -pady 2 -padx 2 -columnspan 3
	grid .einstellungen.lbFrage -row 2 -column 11 -sticky w -pady 2 -padx 2 -columnspan 3

	grid .einstellungen.lbAbstand1 -row 3 -column 6 -padx 20
	grid .einstellungen.lbAbstand2 -row 3 -column 10 -padx 20
	for {set i 0} {$i <= 3} {incr i} {
		set j [expr $i + 3]
		grid .einstellungen.cbTyp${i}JaNein -row $j -column 0 -sticky w -pady 2 -padx 2
		grid .einstellungen.lbTyp${i}Von -row $j -column 1 -sticky e -pady 2 -padx 2
		grid .einstellungen.enTyp${i}Von -row $j -column 2 -sticky w -pady 2 -padx 2
		grid .einstellungen.lbTyp${i}Bis -row $j -column 3 -sticky w -pady 2 -padx 2
		grid .einstellungen.enTyp${i}Bis -row $j -column 4 -sticky w -pady 2 -padx 2
		
		if {$i >= 2} {
			grid .einstellungen.cbTyp${i}Variante -row $j -column 5 -sticky w -pady 2 -padx 2
		}
		
		grid .einstellungen.lbTyp${i}Zeit -row $j -column 7 -sticky w -pady 2 -padx 2
		grid .einstellungen.enTyp${i}Zeit -row $j -column 8 -sticky w -pady 2 -padx 2
		grid .einstellungen.lbTyp${i}ZeitSekunden -row $j -column 9 -sticky w -pady 2 -padx 2
		grid .einstellungen.cbTyp${i}Zahl1 -row $j -column 11 -sticky w -pady 2 -padx 2
		grid .einstellungen.cbTyp${i}Zahl2 -row $j -column 12 -sticky w -pady 2 -padx 2
		grid .einstellungen.cbTyp${i}Zahl3 -row $j -column 13 -sticky w -pady 2 -padx 2
	}
	grid .einstellungen.btOK -row 8 -column 0 -pady 2 -padx 2 -columnspan 14
	grid .einstellungen.lbHilfe -row 9 -column 0 -pady 2 -padx 2 -columnspan 14 -sticky w
	bind .einstellungen.combStufe <<ComboboxSelected>> {KonfigurationEinlesen 0 $Glob(StufenMax); EinstellungenWerteSetzen}

	tkwait visibility .einstellungen
	grab set .einstellungen
}

proc ErgebnisPruefen {} {
	# prüft das Ergebnis
	
	global Glob ; # lesender und schreibender Zugriff

	# löscht alle noch nicht ausgeführten, aber mit after eingeplanten Prozeduren
	foreach Element [after info] {
		after cancel $Element
	}
    
    if {[.frAufgabe.enZahl1 cget -state] == "normal"} {
		set Eingabe $Glob(Zahl1)
	} 
	if {[.frAufgabe.enZahl2 cget -state] == "normal"} {
		set Eingabe $Glob(Zahl2)
	} 
	if {[.frAufgabe.enZahl3 cget -state] == "normal"} {
		set Eingabe $Glob(Zahl3)
	} 

    .frAufgabe.enZahl1 configure -state readonly
    .frAufgabe.enZahl2 configure -state readonly
    .frAufgabe.enZahl3 configure -state readonly
    
    
	if {$Glob(NaechsteAufgabe) == 0} {
		set Glob(FigurComputerStopp) 1
		set Glob(BalkenStopp) 1

		if {$Eingabe == $Glob(Loesung)} {
			incr Glob(AnzahlRichtig)
			set Glob(TextErgebnis) [::msgcat::mc "Richtig!"]
			update idletask
			after 1000
			FigurBewegen $Glob(FigurSpieler2) $Glob(SchrittweiteFigur) $Glob(Startzeit) $Glob(RechenzeitMax) $Glob(FigurSpielerY)
		} else {
			incr Glob(AnzahlFalsch)
			set Glob(TextErgebnis) [::msgcat::mc "Falsch!"]
			update idletask
			after 1000
			FigurBewegen $Glob(FigurComputer2) $Glob(SchrittweiteFigur) $Glob(Startzeit) $Glob(RechenzeitMax) $Glob(FigurComputerY)
		}
		set Glob(TextPunkte) [PunkteAnzeigen 0 $Glob(AnzahlRichtig)]
		update idletask
		set Glob(NaechsteAufgabe) 1  
	}  
}

proc Animation {} {
	# bewegt die Computerfigur während der Spieler die Aufgabe rechnet
	
	global Glob ; # lesender und schreibender Zugriff

	set AktuelleZeit [clock clicks -milliseconds]
	set Glob(Rechenzeit) [expr $AktuelleZeit - $Glob(Startzeit)]
	if {$Glob(BalkenStopp) == 0 && $Glob(Rechenzeit) <= $Glob(RechenzeitMax)} {
		set x [expr $Glob(xStart)+int($Glob(Rechenzeit)*double($Glob(SchrittweiteFigur)))]
		if {int($x) > int($Glob(xStart))} {
			.can moveto $Glob(FigurComputer2) $x 40
		}
		after 10 Animation
	} else {
		ErgebnisPruefen
	}
}

proc ListeAlteErgebnisseErgaenzen {i} {
	# Hängt die Zahl an die Liste mit den alten Ergebnissen an.
	# Dabei wird geprüft, dass die Liste maximal nur 2/3 soviel Elemente enthält, wie Aufgaben gerechnet werden soll

	# i = Aufgabentyp (0=Plus, 1=Minus, 2=Mal, 3=Geteilt)
	
	global Glob ; # lesender und schreibender Zugriff
	global Konf ; # nur lesender Zugriff

	set AnzahlElemente [llength $Glob(ListeAlteErgebnisse,$i)]
	if {$AnzahlElemente >= [expr int($Konf(Einstellungen,Anzahl)*2.0/3)]} {
		set Glob(ListeAlteErgebnisse,$i) [lrange $Glob(ListeAlteErgebnisse,$i) 1 end]
	}
	switch $i {
		0 {lappend Glob(ListeAlteErgebnisse,0) $Glob(Zahl3)}
		1 {lappend Glob(ListeAlteErgebnisse,1) $Glob(Zahl1)}
		2 {lappend Glob(ListeAlteErgebnisse,2) $Glob(Zahl3)}
		3 {lappend Glob(ListeAlteErgebnisse,3) $Glob(Zahl1)}
	}
}

proc Start {} {
	# Startet das Spiel (nach dem Klick auf den Start-Button)
	
	global Glob ; # lesender und schreibender Zugriff
	global Konf ; # nur lesender Zugriff

	set Glob(TextPunkte) [Aufgabentext $Konf(Einstellungen,Anzahl)]

	.frButton.btStart configure -state disabled

	.can moveto $Glob(FigurSpieler2) 0 $Glob(FigurSpielerY)
	.can moveto $Glob(FigurComputer2) 0 $Glob(FigurComputerY)

	set Glob(AnzahlRichtig) 0
	set Glob(AnzahlFalsch) 0

	while {[expr $Glob(AnzahlRichtig) + $Glob(AnzahlFalsch)] < $Konf(Einstellungen,Anzahl)} {
		set Glob(FigurComputerStopp) 0
		set Glob(NaechsteAufgabe) 0
		set Glob(Rechenzeit) 0
		set Glob(TextErgebnis) ""
	 
		set Rueckgabe [AufgabeStellen]
		lassign $Rueckgabe Glob(Zahl1) Glob(Zahl2) Glob(Zahl3) Glob(RechenzeitMax) Glob(Operator1) Aufgabe
		ListeAlteErgebnisseErgaenzen $Aufgabe
		
		set Rueckgabe [AufgabeFelderFreigeben $Aufgabe $Glob(Zahl1) $Glob(Zahl2) $Glob(Zahl3)]
		lassign $Rueckgabe Glob(Zahl1) Glob(Zahl2) Glob(Zahl3) Glob(Loesung)
		
		.frZeit.pb configure -maximum $Glob(RechenzeitMax)
		
		set Rueckgabe [AnimationsdatenEinstellen  $Glob(Teilstrecke) $Glob(GesamtstreckeMax) $Konf(Einstellungen,Anzahl) $Glob(RechenzeitMax)]
		lassign $Rueckgabe Glob(Gesamtstrecke) Glob(Teilstrecke) Glob(SchrittweiteFigur)
		
		set Glob(BalkenStopp) 0
		set Glob(xStart) [lindex [.can coords $Glob(FigurComputer2)] 0]
		set Glob(xStart) [expr int($Glob(xStart))]
		set Glob(Startzeit) [clock clicks -milliseconds]
		Animation
		vwait Glob(NaechsteAufgabe)
		after 2000
	} 
	set Glob(TextPunkte) [PunkteAnzeigen 1 $Glob(AnzahlRichtig)]
	.frButton.btStart configure -state normal
}

proc PlusAufgabeStellen {Von Bis} {
	# definiert eine Plus-Aufgabe
	
	set Prozent 0.2
	set Faktor 3

	set Spannweite [expr $Bis-$Von+1]
	if {$Spannweite >= 1000} {
		# Einfache Ermittlung der Aufgabe:
		# Nachteil: kleine Zahlen (Null, Eins) kommen zu häufig vor
		set Zahl3 [expr int($Von+($Bis-$Von+1)*rand())]
		set Zahl1 [expr int(($Zahl3+1)*rand())]
	} else {
		# Aufwändige Ermittlung der Aufgabe:
		set Grenze1 [expr round($Von+$Prozent*$Spannweite)]
		set Grenze2 $Bis
		set Zahl3 [ZahlZiehen $Von $Bis $Grenze1 $Grenze2 $Faktor]

		set Von 0
		set Bis $Zahl3
		set Spannweite [expr $Bis-$Von+1]
		set Grenze1 [expr round($Von+$Prozent*$Spannweite)]
		set Grenze2 [expr round($Bis-$Prozent*$Spannweite)]
		set Zahl1 [ZahlZiehen $Von $Bis $Grenze1 $Grenze2 $Faktor]
	}
	set Zahl2 [expr $Zahl3 - $Zahl1]
	
	lappend Rueckgabe $Zahl1 $Zahl2 $Zahl3
	return $Rueckgabe
}

proc MinusAufgabeStellen {Von Bis} {
	# definiert eine Minus-Aufgabe

	set Prozent 0.2
	set Faktor 3

	set Spannweite [expr $Bis-$Von+1]
	if {$Spannweite >= 1000} {
		# Einfache Ermittlung der Aufgabe:
		# Nachteil: kleine Zahlen (Null, Eins) kommen häufig vor
		set Zahl1 [expr int($Von+($Bis-$Von+1)*rand())]
		set Zahl2 [expr int(($Zahl1+1)*rand())]
	} else {
		# Aufwändige Ermittlung der Aufgabe:
		set Grenze1 [expr round($Von+$Prozent*$Spannweite)]
		set Grenze2 $Bis
		set Zahl1 [ZahlZiehen $Von $Bis $Grenze1 $Grenze2 $Faktor]

		set Von 0
		set Bis $Zahl1
		set Spannweite [expr $Bis-$Von+1]
		set Grenze1 [expr round($Von+$Prozent*$Spannweite)]
		set Grenze2 [expr round($Bis-$Prozent*$Spannweite)]
		set Zahl2 [ZahlZiehen $Von $Bis $Grenze1 $Grenze2 $Faktor]
	}
    set Zahl3 [expr $Zahl1 - $Zahl2]
	
	lappend Rueckgabe $Zahl1 $Zahl2 $Zahl3
	return $Rueckgabe
}

proc MalAufgabeStellen {Von Bis Variante} {
	# definiert eine Mal-Aufgabe

	set Prozent 0.2
	set Faktor 10

	set Wurzel [expr int(sqrt($Bis))]
	if {$Variante == 0} {
		set Von2 [expr int($Von/$Wurzel)]
		set Bis2 $Wurzel
		set Spannweite [expr $Bis2-$Von2+1]
		if {$Spannweite >= 1000} {
			set ZahlVon [expr int($Von/$Wurzel)]
			set Zufallszahl1 [expr $ZahlVon+int(($Wurzel-$ZahlVon+1)*rand())]
		} else {
			set Grenze1 [expr round($Von2+$Prozent*$Spannweite)]
			set Grenze2 [expr round($Bis2-$Prozent*$Spannweite)]
			set Zufallszahl1 [ZahlZiehen $Von2 $Bis2 $Grenze1 $Grenze2 $Faktor]
		}
	} else {
		set Von2 0
		set Bis2 $Wurzel
		set Spannweite [expr $Bis2-$Von2+1]
		if {$Spannweite >= 1000} {
			set Zufallszahl1 [expr int(($Wurzel+1)*rand())]
		} else {
			set Grenze1 [expr round($Von2+$Prozent*$Spannweite)]
			set Grenze2 [expr round($Bis2-$Prozent*$Spannweite)]
			set Zufallszahl1 [ZahlZiehen $Von2 $Bis2 $Grenze1 $Grenze2 $Faktor]
		}
	}

	set ListeFaktoren {}
	set i 1
	
	if {$Variante == 0} {
		set Obergrenze $Wurzel
	} else {
		set Obergrenze $Bis
	}

	for {set i 1} {$i <= $Obergrenze} {incr i} {
		set Vielfaches [expr $i * $Zufallszahl1]
		if {$Vielfaches >= $Von && $Vielfaches <= $Bis} {
			lappend ListeFaktoren $i
		}
	}
	
	set Von2 [lindex $ListeFaktoren 0]
	set Bis2 [lindex $ListeFaktoren end]
	set Spannweite [expr $Bis2-$Von2+1]
	if {$Spannweite >= 1000} {
		set AnzahlElemente [llength $ListeFaktoren]
		if {$AnzahlElemente > 0} {
			set Index [expr int($AnzahlElemente*rand())]
			set Zufallszahl2 [lindex $ListeFaktoren $Index]
		} else {
			set Zufallszahl2 1
		}
	} else {
		set Grenze1 [expr round($Von2+$Prozent*$Spannweite)]
		set Grenze2 $Bis2
		set Zufallszahl2 [ZahlZiehen $Von2 $Bis2 $Grenze1 $Grenze2 $Faktor]
	}
	
	if {[expr int(2*rand())] == 0} {
		set Zahl1 $Zufallszahl1
		set Zahl2 $Zufallszahl2
	} else {
		set Zahl1 $Zufallszahl2
		set Zahl2 $Zufallszahl1
	}
	set Zahl3 [expr $Zahl1 * $Zahl2]

	lappend Rueckgabe $Zahl1 $Zahl2 $Zahl3
	return $Rueckgabe
}

proc GeteiltAufgabeStellen {Von Bis Variante} {
	# definiert eine Geteilt-Aufgabe

	set Prozent 0.2
	set Faktor 10

	set Wurzel [expr int(sqrt($Bis))]
	if {$Variante == 0} {
		set Von2 [expr int($Von/$Wurzel)]
		set Bis2 $Wurzel
		set Spannweite [expr $Bis2-$Von2+1]
		if {$Spannweite >= 1000} {
			set ZahlVon [expr int($Von/$Wurzel)]
			set Zufallszahl1 [expr $ZahlVon+int(($Wurzel-$ZahlVon+1)*rand())]
		} else {
			set Grenze1 [expr round($Von2+$Prozent*$Spannweite)]
			set Grenze2 [expr round($Bis2-$Prozent*$Spannweite)]
			set Zufallszahl1 [ZahlZiehen $Von2 $Bis2 $Grenze1 $Grenze2 $Faktor]
		}
	} else {
		set Von2 1 ; # darf nicht Null sein
		set Bis2 $Wurzel
		set Spannweite [expr $Bis2-$Von2+1]
		if {$Spannweite >= 1000} {
			set Zufallszahl1 [expr 1+int($Wurzel*rand())] ; # darf nicht Null sein
		} else {
			set Grenze1 [expr round($Von2+$Prozent*$Spannweite)]
			set Grenze2 [expr round($Bis2-$Prozent*$Spannweite)]
			set Zufallszahl1 [ZahlZiehen $Von2 $Bis2 $Grenze1 $Grenze2 $Faktor]
		}
	}
	if {$Zufallszahl1 == 0} {
		set Zufallszahl1 1
	}

	if {$Variante == 0} {
		set Obergrenze $Wurzel
	} else {
		set Obergrenze $Bis
	}

	set ListeFaktoren {}
	set i 1
	for {set i 1} {$i <= $Obergrenze} {incr i} {
		set Vielfaches [expr $i * $Zufallszahl1]
		if {$Vielfaches >= $Von && $Vielfaches <= $Bis} {
			lappend ListeFaktoren $i
		}
	}

	set Von2 [lindex $ListeFaktoren 0]
	set Bis2 [lindex $ListeFaktoren end]
	set Spannweite [expr $Bis2-$Von2+1]
	if {$Spannweite >= 1000} {
		set AnzahlElemente [llength $ListeFaktoren]
		if {$AnzahlElemente > 0} {
			set Index [expr int($AnzahlElemente*rand())]
			set Zufallszahl2 [lindex $ListeFaktoren $Index]
		} else {
			set Zufallszahl2 1
		}
	} else {
		set Grenze1 [expr round($Von2+$Prozent*$Spannweite)]
		set Grenze2 $Bis2
		set Zufallszahl2 [ZahlZiehen $Von2 $Bis2 $Grenze1 $Grenze2 $Faktor]
	}

	if {[expr int(2*rand())] == 0} {
		set Zahl2 $Zufallszahl1
		set Zahl3 $Zufallszahl2
	} else {
		set Zahl2 $Zufallszahl2
		set Zahl3 $Zufallszahl1
	}
	set Zahl1 [expr $Zahl2 * $Zahl3]
	lappend Rueckgabe $Zahl1 $Zahl2 $Zahl3
	return $Rueckgabe
}

proc AufgabeStellen {} {
	# definiert eine Aufgabe vom Typ Plus, Minus, Mal oder Geteilt
	
	global Konf ; # Zugriff erfolgt nur lesend
	global Glob ; # Zugriff erfolgt nur lesend
	
	# Augabentyp Plus, Minus, Mal, Geteilt zufällig auswählen
	set OK 0
	while {$OK == 0} {
		set Aufgabe [expr int(4*rand())]
		if {($Aufgabe == 0 && $Konf(Einstellungen,0,JaNein) == 1) || ($Aufgabe == 1 && $Konf(Einstellungen,1,JaNein) == 1) || ($Aufgabe == 2 && $Konf(Einstellungen,2,JaNein) == 1) || ($Aufgabe == 3 && $Konf(Einstellungen,3,JaNein) == 1)} {
			set OK 1
		}
	}
	
	switch $Aufgabe {
		0	{
				set Zahl3 [lindex $Glob(ListeAlteErgebnisse,0) 0] ; # es wird immer das erste Element der Liste zugeordnet, damit die while-Schleife mindestens einmal durchlaufen wird
				set MaxSchleifen 0
				while {[lsearch $Glob(ListeAlteErgebnisse,0) $Zahl3] >= 0 && $MaxSchleifen<=50} {
					set Rueckgabe [PlusAufgabeStellen $Konf(Einstellungen,0,Von) $Konf(Einstellungen,0,Bis)]
					lassign $Rueckgabe Zahl1 Zahl2 Zahl3
					incr MaxSchleifen
				}
				set Operator1 "+"
				set RechenzeitMax [expr $Konf(Einstellungen,0,Zeit)*1000]
			}
		1	{
				set Zahl1 [lindex $Glob(ListeAlteErgebnisse,1) 0] ; # es wird immer das erste Element der Liste zugeordnet, damit die while-Schleife mindestens einmal durchlaufen wird
				set MaxSchleifen 0
				while {[lsearch $Glob(ListeAlteErgebnisse,1) $Zahl1] >= 0 && $MaxSchleifen<=50} {
					set Rueckgabe [MinusAufgabeStellen $Konf(Einstellungen,1,Von) $Konf(Einstellungen,1,Bis)]
					lassign $Rueckgabe Zahl1 Zahl2 Zahl3
					incr MaxSchleifen
				}
				set Operator1 "-"
				set RechenzeitMax [expr $Konf(Einstellungen,1,Zeit)*1000]
			}
		2	{
				set Zahl3 [lindex $Glob(ListeAlteErgebnisse,2) 0] ; # es wird immer das erste Element der Liste zugeordnet, damit die while-Schleife mindestens einmal durchlaufen wird
				set MaxSchleifen 0
				while {[lsearch $Glob(ListeAlteErgebnisse,2) $Zahl3] >= 0 && $MaxSchleifen<=50} {
					set Rueckgabe [MalAufgabeStellen $Konf(Einstellungen,2,Von) $Konf(Einstellungen,2,Bis) $Konf(Einstellungen,2,Variante)]
					lassign $Rueckgabe Zahl1 Zahl2 Zahl3
					incr MaxSchleifen
				}
				set Operator1 "*"
				set RechenzeitMax [expr $Konf(Einstellungen,2,Zeit)*1000]
			}
		3	{
				set Zahl1 [lindex $Glob(ListeAlteErgebnisse,3) 0] ; # es wird immer das erste Element der Liste zugeordnet, damit die while-Schleife mindestens einmal durchlaufen wird
				set MaxSchleifen 0
				while {[lsearch $Glob(ListeAlteErgebnisse,3) $Zahl1] >= 0 && $MaxSchleifen<=50} {
					set Rueckgabe [GeteiltAufgabeStellen $Konf(Einstellungen,3,Von) $Konf(Einstellungen,3,Bis) $Konf(Einstellungen,3,Variante)]
					lassign $Rueckgabe Zahl1 Zahl2 Zahl3
					incr MaxSchleifen
				}
				set Operator1 "/"
				set RechenzeitMax [expr $Konf(Einstellungen,3,Zeit)*1000]
			}
	}
	lappend Rueckgabe2 $Zahl1 $Zahl2 $Zahl3 $RechenzeitMax $Operator1 $Aufgabe
	return $Rueckgabe2
}

# **** Hauptprogramm ****

set Konf(Sprache) "Deutsch" ; # "English"
SpracheEinstellen

# Zufallszahl erzeugen
expr srand(int([clock seconds]))

# **** falls es keine config-Datei gibt, werden die Dialog-Werte gesetzt ****
# Die vier Aufgabenarten Plus, Minus, Mal, Geteilt definieren
for {set i 0} {$i <= 3} {incr i} {
	set Konf(Einstellungen,$i,JaNein) 1
	set Konf(Einstellungen,$i,Von) 1
	set Konf(Einstellungen,$i,Bis) 10
	set Konf(Einstellungen,$i,Variante) 0
	set Konf(Einstellungen,$i,FrageNachZahl1) 0
	set Konf(Einstellungen,$i,FrageNachZahl2) 0
	set Konf(Einstellungen,$i,FrageNachZahl3) 1
	set Konf(Einstellungen,$i,Zeit) 40
}
# die Aufgaben Minus, Mal, Geteilt werden deaktiviert, nur die Plus-Aufgabe bleibt aktiviert
for {set i 1} {$i <= 3} {incr i} {
	set Konf(Einstellungen,$i,JaNein) 0
}
# 
set Konf(Einstellungen,Anzahl) 10
# Alle Spielstufen werden mit denselben Werten initialisiert
for {set Konf(Einstellungen,Stufe) 1} {$Konf(Einstellungen,Stufe) <= $Glob(StufenMax)} {incr Konf(Einstellungen,Stufe)} {
	EinstellungenWerteUebernehmen
}
# ****

for {set i 0} {$i <= 3} {incr i} {
	lappend Glob(ListeAlteErgebnisse,$i) 0 ; # die Liste darf nicht leer sein
}
set Konf(Einstellungen,Stufe) 1
KonfigurationEinlesen 1 $Glob(StufenMax)
EinstellungenWerteSetzen
SpracheEinstellen
set Glob(TextPunkte) [Aufgabentext $Konf(Einstellungen,Anzahl)]

set Rueckgabe [AnimationsdatenEinstellen $Glob(Teilstrecke) $Glob(GesamtstreckeMax) $Konf(Einstellungen,Anzahl) $Glob(RechenzeitMax)]
lassign $Rueckgabe Glob(Gesamtstrecke) Glob(Teilstrecke) Glob(SchrittweiteFigur)

wm title . $ProgrammTitel 
wm minsize . 350 250
wm resizable . 0 0

option add *Menu.tearOff 0
. configure -menu .m
menu .m
.m add command -label [::msgcat::mc "Einstellungen"] -command {DialogEinstellungen}
.m add cascade -label [::msgcat::mc "Sprache"] -menu .m.sprache
menu .m.sprache
.m.sprache add radiobutton -label "Deutsch" -variable Konf(Sprache) -command {SpracheAendern}
.m.sprache add radiobutton -label "English" -variable Konf(Sprache) -command {SpracheAendern}
.m add command -label [::msgcat::mc "Info"] -command {InfoAnzeigen $ProgrammName $ProgrammVersion}

ttk::frame .frAufgabe
ttk::entry .frAufgabe.enZahl1 -textvariable Glob(Zahl1) -width 5 -state readonly -font MeineSchrift
ttk::entry .frAufgabe.enZahl2 -textvariable Glob(Zahl2) -width 5 -state readonly -font MeineSchrift
ttk::entry .frAufgabe.enZahl3 -textvariable Glob(Zahl3) -width 5 -state readonly -font MeineSchrift
ttk::label .frAufgabe.lbOperator1 -textvariable Glob(Operator1)
ttk::label .frAufgabe.lbOperator2 -textvariable Glob(Operator2)

ttk::label .frAufgabe.lbErgebnis -textvariable Glob(TextErgebnis)

ttk::frame .frZeit
ttk::label .frZeit.lb -text [::msgcat::mc "Zeit:"]
ttk::progressbar .frZeit.pb -orient horizontal -maximum $Glob(RechenzeitMax) -length 180 -variable Glob(Rechenzeit)

ttk::frame .frButton
ttk::button .frButton.btStart -text [::msgcat::mc "Start"] -command Start
ttk::button .frButton.btEnde -text [::msgcat::mc "Ende"] -command Programmende

set TextPunkte [Aufgabentext $Konf(Einstellungen,Anzahl)]
ttk::label .lbPunkte -textvariable Glob(TextPunkte)

image create photo FigurSpieler -file [file join [file dirname $Konf(Skriptname)] FigurSpieler.ppm]
image create photo FigurComputer -file [file join [file dirname $Konf(Skriptname)] FigurComputer.ppm]
image create photo FigurZiel -file [file join [file dirname $Konf(Skriptname)] FigurZiel.ppm]

canvas .can -width [expr $Glob(Gesamtstrecke) + 80] -height 75 -background white ;
set Glob(FigurSpieler2) [.can create image 0 $Glob(FigurSpielerY) -anchor nw -image FigurSpieler]
set Glob(FigurComputer2) [.can create image 0 $Glob(FigurComputerY) -anchor nw -image FigurComputer]
set Glob(FigurZiel2) [.can create image [expr $Glob(Gesamtstrecke) + 40] $Glob(FigurZielY) -anchor nw -image FigurZiel]

pack .frAufgabe -pady 10 -padx 10 -anchor w
grid .frAufgabe.enZahl1 -column 0 -row 0
grid .frAufgabe.lbOperator1 -column 1 -row 0 -padx 2
grid .frAufgabe.enZahl2 -column 2 -row 0
grid .frAufgabe.lbOperator2 -column 3 -row 0 -padx 2
grid .frAufgabe.enZahl3 -column 4 -row 0
grid .frAufgabe.lbErgebnis -column 5 -row 0

pack .frZeit -pady 10 -padx 10 -anchor w
grid .frZeit.lb -column 0 -row 0
grid .frZeit.pb -column 1 -row 0

pack .can -padx 10 -anchor w
pack .lbPunkte -pady 10 -padx 10
pack .frButton -pady 10 -padx 10 -anchor w
grid .frButton.btStart -column 0 -row 0
grid .frButton.btEnde -column 1 -row 0

bind . "<Return>" ErgebnisPruefen   
bind . "<KP_Enter>" ErgebnisPruefen   
 
